### SD-Card: create partitions and fileysytems
#
# Created by: Davide Madrisan
# Modified by: Silvan Calarco <silvan.calarco@mambasoft.it>
#
DRIVE=$1
ROOTFSARCHIVE=$2
[ "$3" = "-y" ] && ASSUMEYES=1

[ "$DRIVE" -a "$ROOTFSARCHIVE" ] || {
   echo "Usage: $0 device rootfs-archive [-y]"
   echo
   echo "-y: assume yes to all questions (script mode)"
   echo
   echo "Example: $0 /dev/sdb /tmp/openmamba-rootfs-light.arm.tar.xz"
   exit 1
}

[ $UID -eq 0 ] || {
   echo "ERROR: this script must be called as root; aborting."
   exit 1
}

if [ "${DRIVE:0:11}" = "/dev/mmcblk" ]; then
   PARTITION1=${DRIVE}p1
   PARTITION2=${DRIVE}p2
else
   PARTITION1=${DRIVE}1
   PARTITION2=${DRIVE}2
fi

[ "$ASSUMEYES" ] || {
   echo -n "All current data in ${DRIVE} will be lost, do you want to proceed [y/N]?"
   read ans
   [ "$ans" = "y" -o "$ans" = "Y" ] || exit 0
}

sudo parted -s ${DRIVE} mktable msdos
if [ "$?" != "0" ]; then
   echo "ERROR: parted exited with error $? while creating partition table on ${DRIVE}; aborting."
fi
# FAT partition
sudo parted -s -a cyl -- ${DRIVE} mkpart primary fat32 0 40M
if [ "$?" != "0" ]; then
   echo "ERROR: parted exited with error $? while creating boot partition on ${DRIVE}; aborting."
fi
# openmamba partition
sudo parted -s -a cyl -- ${DRIVE} mkpart primary ext4 40M -1
if [ "$?" != "0" ]; then
   echo "ERROR: parted exited with error $? while creating system partition on ${DRIVE}; aborting."
fi

echo "Formatting $PARTITION1..."
mkfs.vfat -s 1 -F 32 -n "BOOT" $PARTITION1 || {
   echo "ERROR: unable to format $PARTITION1; aborting."
   exit 1
}

echo "Formatting $PARTITION2..."
mke2fs -q -t ext4 -j -L "openmamba" $PARTITION2 || {
   echo "ERROR: unable to format $PARTITION2; aborting."
   exit 1
}

### SD-Card: install software

echo "Fetching firmware from git..."
SWHOME=`mktemp -d`

cd $SWHOME
git clone git://github.com/raspberrypi/firmware.git --depth=1

BOOTMP=/media/BOOT
ROOTMP=/media/openmamba

[ -e $BOOTMP -o -e $ROOTMP ] && {
   echo "ERROR: /media/BOOT or /media/openmamba already exist."
   exit 1
}

udisks --mount $PARTITION1 || {
   echo "ERROR: unable to mount partition $PARTITION1; aborting."
   exit 1
}
udisks --mount $PARTITION2 || {
   echo "ERROR: unable to mount partition $PARTITION2; aborting."
   exit 1
}

# bootcode.bin: 2nd stage bootloader, starts with SDRAM disabled
# loader.bin  : 3rd stage bootloader, starts with SDRAM enabled
# start.elf   : The GPU binary firmware image, provided by the foundation
# kernel.img  : The OS kernel to load on the ARM processor.
for f in bootcode.bin fixup_x.dat kernel.img start_x.elf LICENCE.broadcom; do
   cp $SWHOME/firmware/boot/$f $BOOTMP/
done

# cmdline.txt : parameters passed to the kernel on boot
echo "\
dwc_otg.lpm_enable=0 \
root=/dev/mmcblk0p2 \
rootfstype=ext4 \
quiet \
rootwait \
splash" > $BOOTMP/cmdline.txt

# Create the kernel configuration file
echo "\
arm_freq=800
force_turbo=1
gpu_mem=100
disable_overscan=1
start_file=start_x.elf
fixup_file=fixup_x.dat
framebuffer_depth=32
framebuffer_ignore_alpha=1" > $BOOTMP/config.txt

## Enable HDMI mode rather than DVI, if required
#echo "\
#hdmi_drive=2" >> $BOOTMP/config.txt

# optional files:
#   config.txt    : A configuration file read by the GPU.
#                   Use this to override set the video mode, alter system clock
#                   speeds, voltages, etc.
#   vlls directory: Additional GPU code, e.g. extra codecs. 
#                   Not present in the initial release.

pushd $ROOTMP >/dev/null
echo "Uncompressing $ROOTFSARCHIVE..."
tar xf $ROOTFSARCHIVE
popd >/dev/null

# copy kernel modules
mkdir -p $ROOTMP/lib/modules
cp -a $SWHOME/firmware/modules/* $ROOTMP/lib/modules/

## swhitch to runlevel 1
#sed -i "s@^id:[0-9]:\(.*\)@id:1:\1@" \
#   $ROOTMP/etc/inittab
## remove root password
## default: root:$1$7QJBkeh1$Dd.G4kEYQT2c6x4kLMrAS.:15418:0:99999:7:::0
#sed -i "s@^root:[^:]*\(.*\)@root:\1@" \
#   $ROOTMP/etc/shadow
#
## /proc/bus/usb/ does not exist
#sed -i "s@^usbfs.*@#&@" $ROOTMP/etc/fstab
#
#sed -i "\
#s@^multithread=1@multithread=0@
#s@#no_auto_depmod.*@no_auto_depmod=1@" \
#   $ROOTMP/etc/sysconfig/rc

sync
udisks --unmount $PARTITION1
udisks --unmount $PARTITION2

[ "$SWHOME" -a "${SWHOME:0:4}" = "/tmp" ] && rm -rf $SWHOME

echo "Done."
exit 0
